/**
 *
 * parse_alioth.c
 *
 * This is Free Software. You can use, modify, and redistribute it
 * under the terms of the GNU General Public License, either version 3
 * of the License or, at your option, any later version.
 *
 *
 * (c) 2017 Vincenzo (KatolaZ) Nicosia 
 *
 * Parse a cgit html file containing a list of repos. The code looks
 * for a <td> tag of class "sublevel-repo", which will contain a tag
 * <a> whose "title" attribute is the name of the repo, and whose
 * "href" attribute is the corresponding git link (relative to
 * ALIOTH_PREFIX)
 *
 *
 *
 */

#include <stdio.h>
#include "expat.h"
#include <string.h>
#include <stdlib.h>


#ifdef XML_LARGE_SIZE
#if defined(XML_USE_MSC_EXTENSIONS) && _MSC_VER < 1400
#define XML_FMT_INT_MOD "I64"
#else
#define XML_FMT_INT_MOD "ll"
#endif
#else
#define XML_FMT_INT_MOD "l"
#endif

#define STATE_IGNORE       0x00
#define STATE_REPO       0x01


/**
 * FIXME!!! This is currently a constant, but we might want to allow
 * the parsing of other cgit installations
 */
#define ALIOTH_PREFIX "https://anonscm.debian.org"

typedef struct {
  unsigned char state;
  char repo_name[256];
  char repo_href[256];
} t_userdata;




char *convert_state(unsigned char state){

  switch(state){
  case 0x00: 
    return "IGNORE";
    break;
  case 0x01: 
    return "REPO";
    break;
  default:
    return "UNDEF";
  }
}


void  dump_repo(t_userdata *data){
  printf("%s: %s\n", data->repo_name, data->repo_href);
}


void update_state_start(const char *name, t_userdata *data, const char **attrs){
  int i = 0;

  //printf("starting elem: %s\n", name);
  if (!strcasecmp(name, "td")){
    while(attrs[i]){
      if (!strcasecmp(attrs[i], "class") && !strcasecmp(attrs[i+1], "sublevel-repo")){
        data->state = STATE_REPO;
      }
      i += 2;
    }
  }
  else if ( data -> state == STATE_REPO && ! strcasecmp(name, "a")){
    while(attrs[i]){
      //printf("key: %s value: %s\n", attrs[i], attrs[i+1]);
      if (!strcasecmp(attrs[i], "title")){
		sprintf(data->repo_name, "%s", attrs[i+1]);
	  }
	  else if (!strcasecmp(attrs[i], "href")){
		sprintf(data->repo_href, "%s%s", ALIOTH_PREFIX, attrs[i+1]);
	  }
	  i += 2;
    }
  }
}

void update_state_end(const char *name, t_userdata *data){

  //printf("ending elem: %s\n", name);
  
  if (!strcasecmp(name, "td") && data->state == STATE_REPO){
    dump_repo(data);
    data -> state = STATE_IGNORE;
  }
}


static void XMLCALL 
textContent(void *userData, const XML_Char *s, int len){

  static char str[8192];
  int dim;
  t_userdata *data = (t_userdata *)userData;
  
  dim = len;
  memcpy(str, s, dim);
  str[dim] = '\0';
  
}

static void XMLCALL
startElement(void *userData, const char *name, const char **attrs)
{
  int i;
  t_userdata *data = (t_userdata *)userData;
  update_state_start(name, data, attrs);
  //printf("open tag: %s state: %s\n", name, convert_state(data -> state));
  //if (data -> state != STATE_IGNORE)
  //  manage_state(name, data, attrs);
}

static void XMLCALL
endElement(void *userData, const char *name)
{
  t_userdata *data = (t_userdata *)userData;
  update_state_end(name, data);
}





int
main(int argc, char *argv[])
{
  char buf[BUFSIZ];
  XML_Parser parser = XML_ParserCreate(NULL);
  int done;
  FILE * fin;
  t_userdata data;

  if (argc == 1){
	fin = stdin;
  }
  else{
	fin = fopen(argv[1], "r");
  }
  
  if (!fin){
    printf("Error opening file %s -- Exiting!!!\n", argv[1]);
    exit(1);
  }

  
  XML_SetUserData(parser, &data);
  XML_SetElementHandler(parser, startElement, endElement);
  XML_SetCharacterDataHandler(parser, textContent);
  do {
    int len = (int)fread(buf, 1, sizeof(buf), fin);
    done = len < sizeof(buf);
    if (XML_Parse(parser, buf, len, done) == XML_STATUS_ERROR) {
      fprintf(stderr,
              "%s at line %" XML_FMT_INT_MOD "u\n",
              XML_ErrorString(XML_GetErrorCode(parser)),
              XML_GetCurrentLineNumber(parser));
      return 1;
    }
  } while (!done);
  XML_ParserFree(parser);
  return 0;
}
